#forkongithub{
    position:fixed;
    display:block;
    top:0;
    right:0;
    width:85px;
    overflow:hidden;
    height:85px;
    z-index:9999;
}
#forkongithub a{
    background:#000;
    color:#fff;
    text-decoration:none;
    font-family:arial,sans-serif;
    text-align:center;
    font-weight:bold;
    font-size:0.55rem;
    line-height:1.35rem;
    transition:0.5s;
    width:125px;
    position:absolute;
    top:23px;
    right:-30px;
    transform:rotate(45deg);
    -webkit-transform:rotate(45deg);
    -ms-transform:rotate(45deg);
    -moz-transform:rotate(45deg);
    -o-transform:rotate(45deg);
    box-shadow:2px 2px 5px rgba(0,0,0,0.8);
}
#forkongithub a:hover{
    background:#c11;
    color:#fff;
}
#forkongithub a::before,#forkongithub a::after{
    content:"";
    width:100%;
    display:block;
    position:absolute;
    top:1px;
    left:0;
    height:1px;
    background:#fff;
}
#forkongithub a::after{
    bottom:1px;
    top:auto;
}